# -*- coding: utf-8 -*-
"""Main modules.

Start from main()
"""
import configparser
import os.path
import sys

from configobj import ConfigObj

from kaidoku.command import command
from kaidoku.help import welcommessage
from kaidoku.misc import openappend

# Kaidoku starts here


def main(argv=sys.argv[1:]):
    """Main module."""
    # Read system configuration from data/system.ini
    inifile = configparser.ConfigParser()
    here = os.path.abspath(os.path.dirname(__file__))
    inifile.read(os.path.join(here, 'data/system.ini'))
    version = inifile.get('system', 'version')
    ConfFile = os.path.expanduser(inifile.get('file', 'config'))

    # Read user configuration
    config = readconfig(ConfFile)
    # configuration file can be redirected
    while 'redirect' in config:
        ConfFile = os.path.expanduser(config['redirect'])
        config = readconfig(ConfFile)

    # Kaidoku command line
    if (len(sys.argv)) == 1:
        print(welcommessage(version))
        while True:
            try:
                arg = input('kaidoku-' + version + '> ').strip().split()
            except Exception:
                print('\nQuitting kaidoku.')
                return
            if len(arg) > 0:
                if arg[0] == 'q':
                    print('Quitting kaidoku.')
                    return
                config = command(arg, config)
                config.write()
    else:
        arg = sys.argv[1:]
        config = command(arg, config)
        config.write()
        return


def readconfig(ConfFile):
    """Read configuration."""
    from PIL import ImageFont

    # Read configuration
    config = ConfigObj(os.path.expanduser(ConfFile), encoding='utf-8')

    # Return if it has redirect
    if 'redirect' in config:
        return config

    # Read system default configuration from data/system.ini
    inifile = configparser.ConfigParser()
    here = os.path.abspath(os.path.dirname(__file__))
    inifile.read(os.path.join(here, 'data/system.ini'))
    defaultLevel = inifile.get('default', 'level')
    defaultMaxtime = inifile.get('default', 'maxtime')
    defaultSymmetry = inifile.get('default', 'symmetry')
    defaultMincell = inifile.get('default', 'mincell')
    defaultAxis = inifile.get('default', 'axis')
    fonts = inifile.get('default', 'font').split()

    # Set default configuration
    if 'datadir' in config:
        datadir = os.path.expanduser(config['datadir'])
        if not os.path.isdir(datadir) and datadir != '':
            try:
                os.mkdir(datadir)
            except Exception:
                print('Error: directory cannot be created.', datadir)
                datadir = ''
    else:
        datadir = ''
    config['datadir'] = datadir
    if 'file' in config:
        file = os.path.expanduser(config["file"])
        if os.path.exists(file):
            try:
                input = open(file, 'r')
                input.close()
            except Exception:
                out = openappend(file)
                if out == '':
                    print('Cannot open file:', file)
                    sys.exit()
                out.close()
            config["file"] = file
        else:
            print(file + ' does not exist.')
            file = os.path.abspath(os.path.dirname(__file__)) + '/data/sudoku.txt'
            print('Changing to default file: ' + file)
            config['file'] = file
    else:
        file = os.path.abspath(os.path.dirname(__file__)) + '/data/sudoku.txt'
        config["file"] = file
    if 'file2' in config:
        file2 = os.path.expanduser(config["file2"])
    else:
        if file != '':
            name, ext = os.path.splitext(file)
            file2 = name + '2' + ext
            config["file2"] = file2
        else:
            file2 = ''
    if 'giveup' in config:
        giveup = os.path.expanduser(config["giveup"])
    else:
        giveup = ''
        config["giveup"] = giveup
    if 'level' in config:
        level = config["level"]
    else:
        level = defaultLevel
        config["level"] = level
    if 'maxtime' in config:
        maxtime = config["maxtime"]
    else:
        maxtime = defaultMaxtime
        config["maxtime"] = maxtime
    if 'pointer' in config:
        pointer = config["pointer"]
    else:
        pointer = [''] + [1] * 9
        config["pointer"] = pointer
    if 'move' not in config:
        config["move"] = []
    if 'bookmark' in config:
        bookmark = config["bookmark"]
    else:
        bookmark = {}
        config["bookmark"] = bookmark
    if 'create' not in config:
        config['create'] = {}
    if 'symmetry' not in config['create']:
        config['create']['symmetry'] = defaultSymmetry
    if 'minlevel' not in config['create']:
        config['create']['minlevel'] = 1
    if int(config['create']['minlevel']) < 1:
        config['create']['minlevel'] = 1
    if int(config['create']['minlevel']) > 9:
        config['create']['minlevel'] = 1
    if 'mincell' not in config['create']:
        config['create']['mincell'] = defaultMincell
    if 'figure' not in config:
        config['figure'] = {}
    if 'font' not in config['figure']:
        for font in fonts:
            try:
                ImageFont.truetype(font, int(16))
                config['figure']['font'] = font + '.ttf'
                break
            except Exception:
                continue
            config['figure']['font'] = 'NotFound'
    if 'color' not in config['figure']:
        config['figure']['color'] = 'black'
    if 'lastcolor' not in config['figure']:
        config['figure']['lastcolor'] = 'green'
    if 'normalsize' not in config['figure']:
        config['figure']['normalsize'] = 'medium'
    if 'markedsize' not in config['figure']:
        config['figure']['markedsize'] = 'large'
    if 'axis' not in config['figure']:
        config['figure']['axis'] = defaultAxis
    config.write()
    return config



# -*- coding: utf-8 -*-
"""modules for tests."""


def test_all():
    """do all the tests in this module"""
    test_calc()
    test_command()
    print('Test completed without error.')
    return


def test_command():
    """test modules in calc.py, wing.py, and chain.py."""
    from kaidoku.command import command
    import os
    import sys
    config = {}
    config['datadir'] = ''
    config['file'] = os.path.abspath(
        os.path.dirname(__file__)) + '/data/sudoku.txt'
    config['level'] = 2
    config['maxtime'] = 60
    config['pointer'] = [''] + [1] * 9
    config['move'] = []
    config['bookmark'] = {}
    config['bookmark']['b1'] = {}
    config['bookmark']['b1']['problem'] = '000001058581094302206080010100960820008402100020018500010036709009100005760809031'
    config['bookmark']['b1']['move'] = ''
    config['bookmark']['b1']['added'] = '2017/11/14'
    config['bookmark']['b1']['comment'] = 'XYZ-wing'
    redirect = os.path.abspath(
        os.path.dirname(__file__)) + '/data/redirect'
    stdout = sys.stdout
    for c in [
        ['book', 'Level 1 trivial'],
        ['l 2', 'Level 2 No. 1'],
        ['299', 'Level 2 No. 1: move 1'],
        ['initial', 'Level 2 No. 1\n'],
        ['299', 'Level 2 No. 1: move 1'],
        ['278', 'Level 2 No. 1: move 2'],
        ['592', 'Both R5C8 and R5C9 have the same value of 2.'],
        ['591', 'Level 2 No. 1: move 3'],
        ['i', 'Look at R1C2. What number is available?'],
        ['a 3', '\nLevel 2 No. 1\nValid sudoku with unique solution'],
        ['ac', '\nLevel 2 No. 1: move 3\nValid sudoku with unique solution'],
        ['658', 'Level 2 No. 1: move 4'],
        ['i', 'There is no solution for this position.'],
        ['h', '246 : In the cell of row 2 column 4, put number 6'],
        ['ha', 'a [verb]'],
        ['c', 'Level 2 No. 1: move 4'],
        ['u', 'http://www.sudoku-solutions.com/index.php?puzzle=900850'],
        ['b', 'Level 2 No. 1: move 3'],
        ['j 3', 'Level 2 No. 3'],
        ['n', 'Level 2 No. 4'],
        ['p', 'Level 2 No. 3'],
        ['j 1', 'Level 2 No. 1'],
        ['sp', 'Naked single: R1C2 = 6, R1C6 = 7,'],
        ['l 5', 'Level 5 No. 1'],
        ['sp', 'Naked single: R4C4 = 4'],
        ['ii', 'Following logics are successively used.'],
        ['iii', 'Pointing pair in box 1 removed 5 from R9C3 R7C3'],
        ['bl', '2017/11/14   b1   XYZ-wing'],
        ['br b1', ''],
        ['c', 'XYZ-wing'],
        ['sp', 'Naked single: R8C6 = 7'],
        ['1234', 'Invalid move. If you want to fill'],
        ['iii', '''Pointing pair in box 4 removed 5 from R4C3 
Pointing pair in box 6 removed 6 from R5C8 R6C8 
Naked pair in row 7 made removal from R7C3
Hidden pair in box 7: R7C3 R9C3
XYZ-wing of R3C4 (3, 5, 7) R3C6 (3, 5) R6C4 (3, 7) removes 3 from R1C4.
Pointing triple in box 1 removed 3 from R3C2 
Hidden pair in box 2: R3C4 R3C6
Chain of pairs. If R5C8 is 7 there is no solution because of following chain.
R5C8 (7,9) = 7
R5C5 (5,7) = 5
R4C6 (3,5) = 3
R6C4 (3,7) = 7
R2C4 (6,7) = 6
R2C8 (6,7) = no candidate
Therefore R5C8 should be 9. '''],
        ['solve -4-3---9-,---------,-----4236,7924--3--,----8----,--1--3627,1385-----,--------4,-7---9-6-', '''  1 2 3 4 5 6 7 8 9
 +-----+-----+-----+
1|  4  |3    |  9  |
2|     |     |     |
3|     |    4|2 3 6|
 +-----+-----+-----+
4|7 9 2|4    |3    |
5|     |  8  |     |
6|    1|    3|6 2 7|
 +-----+-----+-----+
7|1 3 8|5    |     |
8|     |     |    4|
9|  7  |    9|  6  |
 +-----+-----+-----+


Invalid sudoku with multiple solutions.'''],
        ['check 003500902082793006900201830240070003830010020617352498021039004300005200008027300 4', '''  1 2 3 4 5 6 7 8 9
 +-----+-----+-----+
1|    3|5    |9   2|
2|  8 2|7 9 3|    6|
3|9    |2   1|8 3  |
 +-----+-----+-----+
4|2 4  |  7  |    3|
5|8 3  |  1  |  2  |
6|6 1 7|3 5 2|4 9 8|
 +-----+-----+-----+
7|  2 1|  3 9|    4|
8|3    |    5|2    |
9|    8|  2 7|3    |
 +-----+-----+-----+

Pointing pair in box 4 removed 5 from R3C3 
Pointing pair in box 4 removed 9 from R8C3 
Pointing pair in box 9 removed 1 from R8C8 R9C8 
Naked pair in row 3 made removal from R3C2
Naked pair in column 9 made removal from R8C9 R9C9
XYZ-wing of R8C5 (4, 6, 8) R7C4 (6, 8) R8C3 (4, 6) removes 6 from R8C4.
XYZ-wing of R7C7 (5, 6, 7) R9C8 (5, 6) R7C1 (5, 7) removes 5 from R7C8.
Trial. If R8C9 is 1,
(1) Naked single: R9C9 = 9
(2) Hidden single in box 7 : R8C2 = 9
(3) Hidden single in box 7 : R7C1 = 7
(4) Hidden single in box 8 : R9C4 = 1
(5) Hidden single in box 9 : R8C8 = 7
(6) Hidden single in box 3 : R3C9 = 7
(7) Naked single: R3C2 = 5, R5C9 = 5
(8) Naked single: R5C3 = 9, R9C2 = 6
(9) Naked single: R1C2 = 7, R4C3 = 5, R8C3 = 4, R9C8 = 5
(10) Contradiction: R9C1 has no candidate.
Therefore R8C9 is 9.'''],
        ['solve 407001008105090040000570300900083000000000206040900000510000000090160800070000030', '''  1 2 3 4 5 6 7 8 9
 +-----+-----+-----+
1|4   7|    1|    8|
2|1   5|  9  |  4  |
3|     |5 7  |3    |
 +-----+-----+-----+
4|9    |  8 3|     |
5|     |     |2   6|
6|  4  |9    |     |
 +-----+-----+-----+
7|5 1  |     |     |
8|  9  |1 6  |8    |
9|  7  |     |  3  |
 +-----+-----+-----+

Valid sudoku with unique solution'''],
        ['solve 034020000800001700000050620000000907001409800409000000098040000002500006000030410 2', '''  1 2 3 4 5 6 7 8 9
 +-----+-----+-----+
1|  3 4|  2  |     |
2|8    |    1|7    |
3|     |  5  |6 2  |
 +-----+-----+-----+
4|     |     |9   7|
5|    1|4   9|8    |
6|4   9|     |     |
 +-----+-----+-----+
7|  9 8|  4  |     |
8|    2|5    |    6|
9|     |  3  |4 1  |
 +-----+-----+-----+

Determine R9C1 by trial.
Search with depth 1 from R5C5.
Valid sudoku with unique solution of level 8 (extreme).'''],
        ['solve 427001008105090040000570300900083000000000206040900000510000000090160800070000030', '''  1 2 3 4 5 6 7 8 9
 +-----+-----+-----+
1|4 2 7|    1|    8|
2|1   5|  9  |  4  |
3|     |5 7  |3    |
 +-----+-----+-----+
4|9    |  8 3|     |
5|     |     |2   6|
6|  4  |9    |     |
 +-----+-----+-----+
7|5 1  |     |     |
8|  9  |1 6  |8    |
9|  7  |     |  3  |
 +-----+-----+-----+

This sudoku has no solution because R8C1 has no candidate.
Invalid sudoku with no solution.''']
    ]:
        com = c[0].split()
        sys.stdout = open(redirect, 'w')
        config = command(com, config)
        sys.stdout = stdout
        infile = open(redirect, 'r')
        out = ''
        for line in infile:
            out += line
        length = len(c[1])
        assert c[1][:length] == out[:length], 'Error of assertion that ' + \
            c[0] + ' begins with ' + c[1] + '\n' + out
    os.remove(redirect)
    return


def test_calc():
    """test modules in calc.py, wing.py, and chain.py.

    Also testing misc.py that are used from these modules
    """
    import copy
    import datetime
    # Following modules are tested
    from kaidoku.calc import hidden
    from kaidoku.calc import hidsing
    from kaidoku.calc import naked
    from kaidoku.calc import naksing
    from kaidoku.calc import pointing
    from kaidoku.calc import possible
    from kaidoku.calc import solve
    from kaidoku.calc import solveone
    from kaidoku.chain import pairchain
    from kaidoku.chain import remotepair
    from kaidoku.misc import box
    from kaidoku.misc import boxlist
    from kaidoku.misc import combmir
    from kaidoku.misc import conv
    from kaidoku.misc import line
    from kaidoku.misc import pairs
    from kaidoku.misc import pbox
    from kaidoku.misc import wingpos
    from kaidoku.wing import xwing
    from kaidoku.wing import xywing
    from kaidoku.wing import xyzwing
    problem = '013003000002040800098000000850007930000504000023900056000000360005090400000600010'
    s, err = conv(problem)
    s2 = copy.copy(s)
    s2, message, level, solved, err = solve(s2, 1, 10, 5)
    assert message == 'Both R1C3 and R1C6 have the same value of 3.', 'Error in solve'
    problem = '010003000002040800098000000850007930000504000023900056000000360005090400000600010'
    s, err = conv(problem)
    assert s[:9] == [0, 1, 0, 0, 0, 3, 0, 0, 0], 'Error in conv'
    s2 = copy.copy(s)
    s2, message, level, solved, err = solve(s2, 1, 10, 5)
    assert message == 'Solved', 'Error in solve'
    p = possible(s)
    assert p[0] == [0, 0, 0, 1, 1, 1, 1, 0, 0], 'Error in possible'
    b = box()
    assert b[0] == [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 18,
                    19, 20, 27, 36, 45, 54, 63, 72], 'Error in box'
    pb = pbox()
    linescan = line()
    assert pb[(0, 0)] == {3, 4, 5, 6, 7, 8}, 'Error in pbox'
    start = datetime.datetime.now()
    endtime = start + datetime.timedelta(seconds=3)
    s, p, message, logic, depth, found, err = solveone(
        s, p, 4, 0, 3, endtime, b, pb, linescan)
    assert message == 'Hidden single in box 2 : R2C6 = 9', 'Error in solveone'
    s, p, message, found, err = naksing(s, p, b, 4)
    assert message == 'Naked single: R2C8 = 7', 'Error in naksing'
    s, err = conv(
        '085002000970601000020000000000500073001203900260004000000000080000409061000700530')
    p = possible(s)
    linescan = line()
    assert linescan[0] == [0, 1, 2, 9, 10, 11, 18, 19, 20], 'Error in line'
    s, p, message, found, err = hidsing(s, p, linescan, 4)
    assert message == 'Hidden single in box 4 : R6C3 = 3', 'Error in hidsing'
    s, err = conv(
        '178500390396000205452093810621384759587169432934275168263450980805900603709030520')
    p = possible(s)
    s, p, logic, message, found, err = pointing(s, p, pb, 4)
    assert message == 'Pointing pair in box 2 removed 7 from R2C6 ', 'Error in pointing'
    s, err = conv(
        '700060050501080000060002008040000007600803905900000020100700090000010206030090001')
    p = possible(s)
    boxl = boxlist(s)
    assert boxl[0] == [1, 2, 10, 18, 20], 'Error in boxlist'
    assert boxl[1] == [3, 5, 12, 14, 21, 22], 'Error in boxlist'
    comb, mirror = combmir(p, boxl)
    s, p, message, found, err = naked(
        s, p, 2, boxl, comb, 4)
    assert message == 'Naked pair in column 9 made removal from R1C9', 'Error in naked'
    s, err = conv(
        '250400068960020000041600200586142379192367080734985612000006800000090056670004023')
    p = possible(s)
    boxl = boxlist(s)
    comb, mirror = combmir(p, boxl)
    assert comb[0] == [(3, 7), (3, 7, 8), (3, 8)], 'Error in combmir'
    assert mirror[0] == [[3, 7, 8], [
        (0, 1, 2), (0, 1), (1, 2)]], 'Error in combmir'
    s, p, message, found, err = naked(
        s, p, 3, boxl, comb, 4)
    assert message == 'Naked triple in box 7 made removal from R7C3 R9C3', 'Error in naked'
    s, err = conv(
        '528641739614937258793500641902000017487195326100000980870403192301209000209010003')
    p = possible(s)
    boxl = boxlist(s)
    comb, mirror = combmir(p, boxl)
    s, p, message, found, err = hidden(
        s, p, 2, boxl, mirror, 4)
    assert message == 'Hidden pair in row 9: R9C2 R9C7', 'Error in hidden'
    s, err = conv(
        '000039760006000004700500010900608030560390002310705006070083001200000803003450000')
    p = possible(s)
    boxl = boxlist(s)
    found = True
    while found:
        comb, mirror = combmir(p, boxl)
        s, p, message, found, err = naked(
            s, p, 2, boxl, mirror, 4)
    s, p, message, found, err = hidden(
        s, p, 3, boxl, mirror, 4)
    assert message == 'Hidden triple in row 2: R2C2 R2C7 R2C8', 'Error in hidden'
    s, err = conv(
        '320456000014200000070008040006020009050000020192080400941500070200801904000940031')
    p = possible(s)
    found = True
    while found:
        s, p, logic, message, found, err = pointing(s, p, pb, 4)
    boxl = boxlist(s)
    comb, mirror = combmir(p, boxl)
    s, p, message, found, err = naked(
        s, p, 2, boxl, comb, 4)
    boxl = boxlist(s)
    comb, mirror = combmir(p, boxl)
    s, p, message, found, err = naked(
        s, p, 4, boxl, comb, 4)
    assert message == 'Naked quad in box 3 made removal from R2C7 R2C9', 'Error in nakquad'
    # Hidden Quads example by Klaus Brenner at http://www.sudokuwiki.org/Hidden_Candidates
    s, err = conv(
        '901500046425090081860010020502000000019000460600000002196040253200060817000001694')
    p = possible(s)
    found = True
    while found:
        s, p, logic, message, found, err = pointing(s, p, pb, 4)
    boxl = boxlist(s)
    comb, mirror = combmir(p, boxl)
    s, p, message, found, err = naked(
        s, p, 2, boxl, comb, 4)
    boxl = boxlist(s)
    comb, mirror = combmir(p, boxl)
    s, p, message, found, err = hidden(
        s, p, 4, boxl, mirror, 4)
    assert message == 'Hidden quad in box 5: R4C4 R4C6 R6C4 R6C6', 'Error in hidquad'
    s, err = conv(
        '030600078270080010500007004027591403090378120103264097300800749040750001700009052')
    p = possible(s)
    boxl = boxlist(s)
    comb, mirror = combmir(p, boxl)
    wing = wingpos(boxl, mirror)
    s, p, message, found, err = xwing(s, p, 2, wing, 4)
    assert (
        message) == 'X-wing of 8 in C1, C8 and R4, R8 removes 8 from R8C7, R8C3', 'Error in xwing'
    s, err = conv(
        '589312746236740005714006023362987050847135269951264387478603500603001078105070630')
    p = possible(s)
    pair, paircomb, pairdict = pairs(s, p)
    s, p, message, found, err = xywing(s, p, b, pair, 4)
    assert (message[:21]) == 'XY-wing of R8C7 (4,9)', 'Error in xywing'
    s, err = conv(
        '240800039073900084800034257064208793907003805308009410732586941481390500600001308')
    p = possible(s)
    boxl = boxlist(s)
    s, p, logic, message, found, err = pointing(s, p, pb, 4)
    assert message == 'Pointing pair in box 2 removed 5 from R1C5 R2C5 ', 'Error in pointing'
    comb, mirror = combmir(p, boxl)
    wing = wingpos(boxl, mirror)
    s, p, message, found, err = xwing(s, p, 2, wing, 4)
    assert (
        message) == 'X-wing of 1 in C2, C4 and R3, R5 removes 1 from R5C5', 'Error in xwing'
    comb, mirror = combmir(p, boxl)
    s, p, message, found, err = xyzwing(s, p, b, boxl, comb, pb, 4)
    assert (
        message[:50]) == 'XYZ-wing of R5C4 (1, 4, 6) R5C5 (4, 6) R3C4 (1, 6)', 'Error in xyzwing'
    s, err = conv(
        '917854623040106007605027104074080016109760042060410970406200701701640230823571469')
    p = possible(s)
    boxl = boxlist(s)
    comb, mirror = combmir(p, boxl)
    wing = wingpos(boxl, mirror)
    s, p, message, found, err = xwing(s, p, 3, wing, 4)
    assert (
        message) == 'Swordfish of 3 in R2, R6, R7 and C1, C5, C6 removes 3 from R4C1, R4C6, R5C6 (=5)'
    comb, mirror = combmir(p, boxl)
    # Jellyfish example from http://www.sudokuwiki.org/Jelly_Fish_Strategy
    s, err = conv(
        '001753800050000007700890100000601570625478931017905400000067004070000010006309700')
    p = possible(s)
    boxl = boxlist(s)
    comb, mirror = combmir(p, boxl)
    wing = wingpos(boxl, mirror)
    s, p, message, found, err = xwing(s, p, 4, wing, 4)
    assert (
        message) == 'Jellyfish of 2 in R1, R4, R6, R9 and C1, C5, C8, C9 removes 2 from R2C1, R2C5, R2C8, R3C8, R3C9, R7C1, R7C8, R8C1, R8C5, R8C9'
    comb, mirror = combmir(p, boxl)
    s, err = conv(

        '037080405658194300402357860063528004504761203201943650829415736006039540345070000')
    p = possible(s)
    boxl = boxlist(s)
    s, p, logic, message, found, err = pointing(s, p, pb, 4)
    assert message == 'Pointing pair in box 2 removed 2 from R1C8 ', 'Error in pointing'
    s, p, logic, message, found, err = pointing(s, p, pb, 4)
    assert message == 'Pointing pair in box 7 removed 1 from R8C9 ', 'Error in pointing'
    comb, mirror = combmir(p, boxl)
    s, p, message, found, err = hidden(
        s, p, 2, boxl, mirror, 4)
    assert message == 'Hidden pair in column 9: R3C9 R9C9', 'Error in hidden'
    pair, paircomb, pairdict = pairs(s, p)
    assert pair[0] == [0, 1, 9], 'Error in pairs.'
    assert paircomb[0] == [0, 1], 'Error in pairs.'
    assert pairdict[19] == [0, 7, 19, 26, 33, 78, 80], 'Error in pairs.'
    s, p, message, found, err = remotepair(
        s, p, b, pair, pairdict, 4)
    assert (
        message[:32]) == 'Remote pairs of (1,9) are found.', 'Error in remotepair'
    s, err = conv(
        '300957800074368015050142000005271639163894050792536100000703590507689420009405006')
    p = possible(s)
    pair, paircomb, pairdict = pairs(s, p)
    s, p, message, chainlength, found, err = pairchain(
        s, p, b, pair, paircomb, 4)
    assert (
        message[:28]) == 'Chain of pairs. If R3C8 is 7', 'Error in pairchain'
    return



import copy
import datetime

from kaidoku.calc import solve
from kaidoku.calc import solveone
from kaidoku.calc import possible
from kaidoku.misc import box
from kaidoku.misc import line
from kaidoku.misc import pbox
from kaidoku.misc import check
from kaidoku.misc import conv
from kaidoku.misc import blank

MAXDEPTH = 20
MAXTIME = 120


class Kaidoku:

    def __init__(self, position):
        self.position = position
        s, err = conv(position)
        self.hints = 0
        if err:
            self.mes = s
            self.valid = False
            return
        self.mes, err = check(s)
        if err:
            self.valid = False
            return
        s2 = copy.deepcopy(s)
        s, self.mes, self.level, found, err = solve(s, 0, MAXDEPTH, MAXTIME)
        s = copy.deepcopy(s2)
        if err:
            self.valid = False
            if found:
                self.mes = 'Multiple solutions to this position'
            else:
                self.mes = 'No solution to this position'
            return
        self.valid = True
        if blank(s) == 0:
            self.mes = 'Already solved'
            return
        start = datetime.datetime.now()
        endtime = start + datetime.timedelta(seconds=MAXTIME)
        p = possible(s)
        b = box()
        pb = pbox()
        linescan = line()
        s, p, self.mes, self.logic, self.depth, found, err = solveone(
            s, p, 4, 0, MAXDEPTH, endtime, b, pb, linescan)
        self.hints = 1
        if self.logic == 'Naked single':
            self.hint = 'Look at {0}. What number is available?'.format(
                self.pos(self.mes[14:18]))
            return
        self.hints = 2
        if self.logic == 'Hidden single':
            mes = self.mes[:self.mes.index(':')]
            self.hint = mes + 'can be found.'
            self.hint2 = mes + 'for ' + self.mes[-1:]
            return
        self.hint = 'Think candidates of the cells.'
        self.hints = 3
        logi = [self.logic]
        mes = [self.mes]
        blank1 = blank(s2)
        while blank(s2) == blank1:
            s2, p, message, logic, depth, found, err = solveone(
                s2, p, 4, 0, blank1, endtime, b, pb, linescan)
            logi.append(logic)
            mes.append(message)
        self.hint2 = 'Use ' + ', '.join(logi) + ' successively.'
        self.hint3 = '\n'.join(mes)

    def pos(self, p):
        return p.replace('R', 'Row:').replace('C', ' Column:')



import tkinter
import copy

class GUIBord(tkinter.Tk):
    def __init__(self):
        super().__init__()

    def display(self,s,move):
        self.title("KAIDOKU")
        self.geometry("270x300")
        self.resizable(0,0)
        self.move = copy.deepcopy(move)

        frame1 = tkinter.Frame(self)
        frame1.grid()

        bordFrame = tkinter.Frame(frame1)
        bordFrame.grid(row=0, column=0)

        # make 3 * 3 blocks
        blockList = [[None]*3 for i in range(3)]
        for i in range(3):
            for j in range(3):
                block = tkinter.Frame(
                    bordFrame,
                    width = 90,
                    height = 90,
                    borderwidth = 1,
                    relief="solid",
                )
                block.grid(row=i, column=j)
                block.grid_propagate(0)
                blockList[i][j] = block
        self.inputList = [[None]*9 for i in range(9)]
        for i in range(9):
            for j in range(9):
                square = tkinter.Frame(
                    blockList[i//3][j//3],
                    width = 30,
                    height = 30,
                    borderwidth = 0,
                    background="white smoke",
                    relief="solid",
                )
                square.propagate(0)
                square.grid(row=i%3, column=j%3)
                # If the value is 0 , create a text box
                if s[i * 9 + j] != 0:
                    label = tkinter.Label(
                        square,
                        text = s[i * 9 + j],
                    )
                    label.pack()
                    self.inputList[i][j] = s[i * 9 + j]
                else:
                    textBox = tkinter.Entry(
                        square,
                        background = "white smoke",
                        justify = "center",
                        validate = 'key',
                        vcmd = (self.register(self.validation),'%P',i,j)
                    )
                    textBox.pack()
                    self.inputList[i][j] = textBox
        for v in move:
            idx = int(v[0]) - 1;jdx = int(v[1]) - 1
            nval = v[2]
            self.inputList[idx][jdx].insert(0,nval)
        # set button
        buttonFrame = tkinter.Frame(frame1)
        buttonFrame.grid(row=1, column=0)
        submitButton = tkinter.Button(
            buttonFrame,
            text="save",
            command=self.saveClick,
        )
        submitButton.pack()
        self.mainloop()

    """When user click save-button"""
    def saveClick(self):
        numList = [[None]*9 for i in range(9)]
        self.move = []
        for i in range(9):
            for j in range(9):
                if type(self.inputList[i][j]) != int:
                    if self.inputList[i][j].get() == '':continue
                    self.move.append(str(i + 1) + str(j + 1) + self.inputList[i][j].get())

    def validation(self,P,idx,jdx):
        idx = int(idx);jdx = int(jdx)
        if len(P) == 0:return True
        if len(P) > 1:return False
        if not (ord('1') <= ord(P) <= ord('9')):return False
        # If input value get stuck in a constraint , return False
        for i in range(9):
            if self.objToStr(self.inputList[idx][i]) == P:return False
            if self.objToStr(self.inputList[i][jdx]) == P:return False
            if self.objToStr(self.inputList[(idx//3)*3+i//3][(jdx//3)*3+i%3]) == P:return False
        return True

    def objToStr(self,obj):
        if type(obj) == int:return str(obj)
        if type(obj) == str:return obj 
        return obj.get()



# -*- coding: utf-8 -*-
"""Modules for creating and maintaining book database."""
import copy
import datetime
import random
from statistics import mean

from kaidoku.calc import possible
from kaidoku.calc import solve
from kaidoku.misc import blank
from kaidoku.misc import box
from kaidoku.misc import check
from kaidoku.misc import conv
from kaidoku.misc import lev
from kaidoku.misc import openappend
from kaidoku.output import short


def analyze(file, level, verbose):
    """Analyze whole database of the level."""
    if file == '':
        return
    start = datetime.datetime.now()
    input = open(file, 'r')
    i = []
    le = []
    n = 0
    for line in input:
        n += 1
        data = line.strip().split(' ')
        if int(data[0]) == level:
            s = conv(data[1])[0]
            s2 = copy.copy(s)
            if verbose > 0:
                print('Solving problem No.{0}'.format(len(i) + 1))
            s2, message, level2, solved, err = solve(s2, verbose, 20, 10)
            if not solved and err:
                print('\nNo solution {0}'.format(data[1]))
            if solved and err:
                print('\nMultiple solutions {0}'.format(data[1]))
            if not solved and not err:
                print('\nGive up {0}'.format(data[1]))
            i.append(blank(s))
            le.append(level2)
            if verbose == 0:
                if len(i) % 10 == 0:
                    print('.', end='', flush=True)
                if len(i) % 200 == 0:
                    dt = datetime.datetime.now() - start
                    t = dt.seconds + float(dt.microseconds) / 1000000
                    print(' {0} problems solved in {1:.1f} seconds (mean: {2:.3f} sec).'.format(
                        len(i), t, t / len(i)))

    dt = datetime.datetime.now() - start
    t = dt.seconds + float(dt.microseconds) / 1000000
    if len(i) == 0:
        input.close
        return
    print('\n{0} problems solved in {1:.1f} seconds (mean: {2:.3f} sec).'.format(
        len(i), t, t / (len(i))))
    for j in range(11):
        n = le.count(j)
        if n > 0:
            print('Level {0}: {1}'.format(j, n))
    print('Numbers: mean {0:.1f} min {1}'.format(
        81 - mean(i), 81 - max(i)))
    input.close
    return


def reanalyze(file, file2):
    """Reanalyze whole database."""
    if file == '':
        return
    start = datetime.datetime.now()
    input = open(file, 'r')
    outfile = open(file2, 'w')
    i = 0
    for line in input:
        data = line.strip().split(' ')
        s = conv(data[1])[0]
        s2 = copy.copy(s)
        s2, message, level, solved, err = solve(s2, 0, 20, 10)
        if not solved and err:
            print('\nNo solution {0}'.format(data[1]))
            continue
        if solved and err:
            print('\nMultiple solutions {0}'.format(data[1]))
            continue
        if not solved and not err:
            print('\nGive up {0}'.format(data[1]))
            continue
        outfile.write(str(level) + ' ' + data[1] + '\n')
        dt = datetime.datetime.now() - start
        t = dt.seconds + float(dt.microseconds) / 1000000
        i += 1
        if i % 10 == 0:
            print('.', end='', flush=True)
        if i % 200 == 0:
            dt = datetime.datetime.now() - start
            t = dt.seconds + float(dt.microseconds) / 1000000
            print(' {0} problems reanalyzed in {1:.1f} seconds (mean: {2:.3f} sec).'.format(
                i, t, t / i))
    input.close
    outfile.close
    print('')
    return


def append_database(file, datadir, giveup, n, creation):
    """Create new problems and append to database."""
    minlevel = int(creation['minlevel'])
    maxtime = 2
    maxdepth = 25

    outfile = openappend(file)
    start = datetime.datetime.now()
    for i in range(n):
        level = 0
        while level < minlevel:
            s, level = create(maxdepth,  maxtime, creation)
            sudoku = short(s)
            if level == 0:
                give = openappend(giveup)
                if give != 'error':
                    give.write('g ' + str(maxtime) + ' ' + sudoku + '\n')
                    give.close()
            else:
                if level >= minlevel:
                    print('.', end='', flush=True)
                    outfile = open(file, 'a')
                    outfile.write(str(level) + ' ' + sudoku + '\n')
                    outfile.flush()
                    outfile.close()
        if (i + 1) % 10 == 0 and i + 1 < n:
            dt = datetime.datetime.now() - start
            t = dt.seconds + float(dt.microseconds) / 1000000
            print(' {0} problems created in {1:.1f} seconds (mean: {2:.3f} sec).'.format(
                i + 1, t, t / (i + 1)))
    dt = datetime.datetime.now() - start
    t = dt.seconds + float(dt.microseconds) / 1000000
    print('\nFinished. {0} problems created in {1:.1f} seconds (mean: {2:.3f} sec).'.format(
        n, t, t / n))
    show_status(file)
    return


def show_status(file):
    """Show status of the database."""
    if file == '':
        return
    input = open(file, 'r')
    li = [0] * 10
    for line in input:
        data = line.strip().split(' ')
        li[int(data[0])] += 1
    su = sum(li)
    if su == 0:
        print('No problem in the book.')
        print('Create problems with c command.')
        return
    for i in range(1, 10):
        print('Level {0} {1:<10}: {2:>6} ({3:5.2f} %)'.format(
            i, lev(i), li[i], li[i] / su * 100))
    print('Total numbers     : {0:>6}'.format(su))
    input.close
    return


def create(maxdepth, maxtime, creation):
    """Create a new problem."""
    if creation['symmetry'] == 'y':
        symmetry = True
    else:
        symmetry = False
    minlevel = int(creation['minlevel'])
    min = int(creation['mincell'])
    order = []
    for i in range(81):
        order.append(i)
    err = True
    while err:
        random.shuffle(order)
        if symmetry:
            order2 = []
            j = 0
            while len(order2) < min:
                while order[j] in order2:
                    j += 1
                order2.append(order[j])
                if order[j] != 40:
                    order2.append(80 - order[j])
            random.shuffle(order2)
        else:
            order2 = order[:min]
        n = []
        v = 0
        while (v < 8):
            for i in range(min):
                n.append(random.randint(1, 9))
                v = len(set(n))
        s = [0] * 81
        for i in range(8):
            s[order2[i]] = i + 1
        for i in range(8, len(order2)):
            p = set(range(1, 10))
            for j in box()[order2[i]]:
                if s[j] in p:
                    p.remove(s[j])
            if len(p) == 0:
                continue
            s[order2[i]] = int(random.sample(p, 1)[0])
        message, err = check(s)
        # if err: print ('err ', end='', flush=True) ##############
        if not err:
            s2 = copy.copy(s)
            s3 = copy.copy(s)
            # Calculate with maxdepth = 999, creating mode
            s2, message, level, solved, err = solve(s2, 0, 999, maxtime)
            if not solved and err:
                return (s3, 0)
            if solved:
                break
            err = True

    while len(s2) == 2:
        p = possible(s)
        if symmetry:
            j = []
            j2 = []
            for i in range(81):
                if s2[0][i] != s2[1][i]:
                    j.append(i)
                    if i < 41 and s2[0][80 - i] != s2[1][80 - i]:
                        j2.append(i)
            if len(j2) > 0:
                j = j2
            random.shuffle(j)
            i = j[0]
        else:
            j = []
            maxp = 0
            for i in range(81):
                if s2[0][i] != s2[1][i]:
                    j.append([i, sum(p[i])])
                    if sum(p[i]) > maxp:
                        maxp = sum(p[i])
            k = []
            for i in range(len(j)):
                if j[i][1] == maxp:
                    k.append(j[i][0])
            random.shuffle(k)
            i = k[0]

        s[i] = s2[1][i]
        if symmetry:
            s[80 - i] = s2[1][80 - i]
        s = shuffle(s)
        s2 = copy.copy(s)
        s2, message, level, solved, err = solve(
            s2, 0, 999, maxtime)  # creating mode
    s2 = copy.copy(s)
    s2, message, level, solved, err = solve(s2, 0, maxdepth, maxtime)

    if level < 3 and minlevel < 3:
        s = make_easy(s, minlevel)
        if blank(s) < 30:
            level = 1

    return (s, level)


def make_easy(s, minlevel):
    """Make a problem easy."""
    s2 = copy.copy(s)
    sol, message, level, solved, err = solve(s2, 0, 1, 1)
    if not solved or err:
        return(s)
    if blank(s) < 25:
        return s
    if blank(s) < 43:
        level = 1
    else:
        if minlevel == 1:
            level = random.randint(1, 2)
        else:
            level = 2
    if level == 1:
        bl = random.randint(24, 28)
    else:
        bl = random.randint(37, 43)
    cell = []
    for i in range(81):
        if s[i] == 0:
            cell.append(i)
    random.shuffle(cell)
    i = 0
    while blank(s) > bl:
        s[cell[i]] = sol[cell[i]]
        s[80 - cell[i]] = sol[80 - cell[i]]
        i += 1

    return s


def shuffle(s):
    """Shuffle board."""
    # rotate right
    s2 = [0] * 81
    for i in range(9):
        for j in range(9):
            s2[j * 9 + 8 - i] = s[i * 9 + j]
    s = copy.copy(s2)
    # mirror 50%
    if random.randint(1, 2) == 1:
        for i in range(9):
            for j in range(9):
                s2[i * 9 + 8 - j] = s[i * 9 + j]
    s = copy.copy(s2)
    n = [1, 2, 3, 4, 5, 6, 7, 8, 9]
    random.shuffle(n)
    for i in range(81):
        if s[i] == 0:
            s2[i] = 0
        else:
            s2[i] = n[s[i] - 1]
    return (s2)


def merge(file, file2):
    """Import file2 to file."""
    if file == '':
        return
    if file2 == '':
        return
    input = open(file, 'r')
    problem = []
    for line in input:
        data = line.strip().split(' ')
        problem.append(data[1])
    input.close
    n = countproblem(file2)
    print('Importing {0} problems.'.format(n))
    input = open(file2, 'r')
    j = 0
    k = 0
    outfile = openappend(file)
    if outfile == 'error':
        print('Unable to write a file:', file)
        return
    for line in input:
        k += 1
        data = line.strip().split(' ')
        if data[1] not in problem:
            j += 1
            outfile.write(line)
    input.close
    outfile.close
    print('{0} / {1} problems appended.'.format(j, k))
    return


def countproblem(file):
    """Count numbers of problems."""
    input = open(file, 'r')
    n = 0
    for line in input:
        data = line.strip().split(' ')
        if not conv(data[1])[1]:
            n += 1
    input.close
    return(n)


def reanalyze_giveup(giveup, t):
    """Reanalyze giveup data."""
    give = givedata(giveup, t)
    for i in give[4]:
        print('Invalid data: ', i)
    if len(give[3]) == 0:
        print('No position to analyze.')
        return
    print('Reanalyzing {0} giveup positions for {1} seconds.'.format(
        len(give[3]), t))
    while True:
        give = givedata(giveup, t)
        if len(give[3]) == 0:
            break
        prob = give[3][0][1]
        err = True
        while err:
            s, err = conv(prob)
            if err:
                give = givedata(giveup, t)
                give[3] = give[3][1:]
                writegive(giveup, give)
        print('Analyzing ... ', end='', flush=True)
        start = datetime.datetime.now()
        dt = datetime.datetime.now() - start
        maxdepth = blank(s)
        s, message, level, solved, err = solve(s, 0, maxdepth, t)
        if solved:
            if err:  # Discard multiple solutions
                give = givedata(giveup, t)
                give[3] = give[3][1:]
                writegive(giveup, give)
                print('multiple solutions. {0} left.'.format(len(give[3])))
            else:
                give = givedata(giveup, t)
                give[3] = give[3][1:]
                give[0].append('u ' + str(dt.seconds) + ' ' + prob)
                writegive(giveup, give)
                print('unique solution. {0} left.'.format(len(give[3])))
        else:
            if err:
                give = givedata(giveup, t)
                give[3] = give[3][1:]
                if dt.seconds > 3:
                    give[1].append('n ' + str(dt.seconds) + ' ' + prob)
                writegive(giveup, give)
                print('no solution. {0} left.'.format(len(give[3])))
            else:
                give = givedata(giveup, t)
                give[3] = give[3][1:]
                give[2].append('g ' + str(t) + ' ' + prob)
                writegive(giveup, give)
                print('give up. {0} left.'.format(len(give[3])))
    return


def givedata(giveup, t):
    """Get giveup data."""
    input = open(giveup, 'r')
    uniq = []
    no = []
    long = []
    short = []
    invalid = []
    for line in input:
        data = line.strip().split(' ')
        if data[0] == 'u':
            uniq.append(line)
            continue
        if data[0] == 'n':
            no.append(line)
            continue
        if data[0] == 'g':
            if conv(data[1])[1]:
                t2 = int(data[1])
                s = data[2]
            else:
                t2 = 3
                s = data[1]
            if t2 < t:
                short.append([t2, s])
                continue
            else:
                long.append(line)
                continue
        invalid.append(line)
    input.close
    g = [uniq, no, long, short, invalid]
    return(g)


def writegive(giveup, g):
    """Write giveup data."""
    try:
        outfile = open(giveup, 'w')
    except Exception:
        print('Unable to write a file:', giveup)
        return
    for i in range(3):
        for j in g[i]:
            outfile.write(j.strip() + '\n')
    for i in g[3]:
        outfile.write('g ' + str(i[0]) + ' ' + i[1] + '\n')
    outfile.close
    return



# -*- coding: utf-8 -*-
"""Chain logic."""


def remotepair(s, p, b, pair, pairdict, verbose):
    """Remote pairs."""
    from kaidoku.misc import cell
    message = ''
    if len(pair) < 4:
        return (s, p, message, False, False)
    for i in pairdict:
        if len(pairdict[i]) > 3:
            pd = pairdict[i]
            link = {}
            for j in pd:
                for k in pd:
                    if k in b[j]:
                        if j in link:
                            link[j].append(k)
                        else:
                            link[j] = [k, ]
            a = []
            c = []
            for j in link:
                if len(link[j]) > 1:
                    a = [j, ]
                    c = link[j]
                    break
            if len(c) > 0:
                appended = True
            else:
                appended = False
            while appended:
                appended = False
                for d in c:
                    for e in link[d]:
                        if e not in a:
                            a.append(e)
                            appended = True
                            for f in link[e]:
                                if f not in c:
                                    c.append(f)
            if len(a) > 1 and len(c) > 1:
                i1 = i // 10 - 1
                i2 = i % 10 - 1
                rpair = []
                for d in a:
                    for e in c:
                        if d not in link[e]:
                            for f in b[d]:
                                if f in b[e] and s[f] == 0:
                                    if p[f][i1] + p[f][i2] > 0:
                                        rpair.append([d, e, f])
                if len(rpair) > 0:
                    if verbose > 2:
                        if verbose == 3:
                            message = 'Remote pairs.'
                        else:
                            message = 'Remote pairs of (' + str(i1+1) + \
                                ',' + str(i2+1) + ') are found.'
                    for rp in rpair:
                        if verbose > 3:
                            message += '\n' + \
                                cell(rp[0]) + ' and ' + \
                                cell(rp[1]) + ' are remote pairs.'
                        p[rp[2]][i1] = 0
                        p[rp[2]][i2] = 0
                        if sum(p[rp[2]]) == 0:
                            if verbose > 3:
                                message += '\nNow ' + \
                                    cell(
                                        rp[2]) + ' has no candidate. This sudoku is unsolvable.'
                            return (s, p, message, False, True)
                        if sum(p[rp[2]]) == 1:
                            s[rp[2]] = p[rp[2]].index(1) + 1
                            if verbose > 3:
                                message += ' ' + \
                                    cell(rp[2]) + ' is ' + str(s[rp[2]]) + '.'
                            return (s, p, message, True, False)
                        else:
                            if verbose > 3:
                                message += ' Candidates of ' + \
                                    cell(rp[2]) + ' changes.'
                    return (s, p, message, True, False)
    return (s, p, message, False, False)


def pairchain(s, p, b, pair, paircomb, verbose):
    """Chain of pairs."""
    from kaidoku.misc import cell
    message = ''
    if len(pair) < 4:
        return (s, p, message, False, False)
    for i in paircomb:
        chain = [i]
        chaincell = [i[0]]
        chain2 = findchain(pair, i, b)
        key = i[0] * 10 + i[1]
        dict = {key: chain2}
        path = {key: [i]}
        if chain2 != []:
            for j in chain2:
                chaincell.append(j[0])
            for j in chain2:
                chain.append(j)
                path[j[0] * 10 + j[1]] = path[i[0] * 10 + i[1]] + [j]
            found = True
            loop = False
            while found:
                found = False
                for j in chain2:
                    jkey = j[0] * 10 + j[1]
                    if jkey in dict:
                        chain3 = dict[jkey]
                    else:
                        chain3 = findchain(pair, j, b)
                        dict[jkey] = chain3
                    for j3 in chain3:
                        path[j3[0] * 10 + j3[1]] = path[jkey] + [j3]
                    if chain3 != []:
                        for j2 in chain3:
                            if j2[0] in chaincell:
                                for k in chain:
                                    if k[0] == j2[0]:
                                        if k[1] == j2[1]:
                                            loop = True
                                        else:
                                            # Found a chain
                                            path1 = path[j2[0] * 10 + j2[1]]
                                            path2 = path[j2[0] * 10 + k[1]]
                                            for i in path1[1:]:
                                                if i in path2:
                                                    p1 = path1.index(i)
                                                    path1 = path1[p1:]
                                                    p2 = path2.index(i)
                                                    path2 = path2[p2:]
                                            i = path1[0]
                                            p[i[0]][i[1] - 1] = 0
                                            n = p[i[0]].index(1) + 1
                                            s[i[0]] = n
                                            chainlength = len(
                                                path1) + len(path2) - 2
                                            if verbose > 2:
                                                if verbose == 3:
                                                    message = 'Chain of pairs. Start from ' + \
                                                        cell(i[0]) + '.'
                                                else:
                                                    # We have path1 and path2 conctradicting chain, and make path circulating chain
                                                    for i in reversed(path2):
                                                        if (i != path2[0]) and (i != path2[-1]):
                                                            for j in pair:
                                                                if i[0] == j[0]:
                                                                    if i[1] == j[1]:
                                                                        k = j[2]
                                                                    else:
                                                                        k = j[1]
                                                                    path1.append(
                                                                        [i[0], k])
                                                                    break
                                                    message = 'Chain of pairs. If ' + \
                                                        cell(i[0]) + ' is ' + str(i[1]) + \
                                                        ' there is no solution because of following chain.\n'
                                                    for i in path1:
                                                        message += cell(i[0]
                                                                        ) + ' ('
                                                        for j in pair:
                                                            if i[0] == j[0]:
                                                                message += str(j[1]) + \
                                                                    ',' + \
                                                                    str(j[2])
                                                                break
                                                        message += ') = '
                                                        if i == path1[-1]:
                                                            message += 'no candidate\n'
                                                        else:
                                                            message += str(i[1]
                                                                           ) + '\n'
                                                    message += 'Therefore ' + cell(path1[0][0]) + \
                                                        ' should be ' + \
                                                        str(n) + '. '
                                            return (s, p, message, chainlength, True, False)
                            chaincell.append(j2[0])
                        if not loop:
                            chain2 = chain3
                            for j in chain2:
                                chain.append(j)
                            found = True

    return (s, p, message, 0, False, False)


def findchain(pair, i, b):
    """Find chain."""
    chain = []
    for j in pair:
        if j[0] in b[i[0]] and (i[1] == j[1] or i[1] == j[2]):
            if i[1] == j[1]:
                chain.append([j[0], j[2]])
            else:
                chain.append([j[0], j[1]])
    return chain



# -*- coding: utf-8 -*-

from setuptools import setup
from codecs import open
from os import path
import configparser

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# Read version from kaidoku/data/system.ini

inifile = configparser.ConfigParser()
inifile.read(path.join(here, 'kaidoku/data/system.ini'))
version = inifile.get('system', 'version')

setup(
    name='kaidoku',
    version=version,
    description='Player, solver and creater of sudoku puzzles',
    long_description=long_description,
    url='https://sekika.github.io/kaidoku/',
    author='Katsutoshi Seki',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: End Users/Desktop',
        'Environment :: Console',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: JavaScript',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows :: Windows 10',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: BSD',
        'Operating System :: POSIX :: Linux',
        'Natural Language :: English',
        'Natural Language :: Japanese',
        'Topic :: Games/Entertainment :: Puzzle Games'
    ],
    keywords='sudoku',
    packages=['kaidoku'],
    package_data={'kaidoku': ['data/*']},
    install_requires=['ConfigObj', 'pillow', 'pyx', 'tk'],
    python_requires=">=3.5",
    entry_points={
        'console_scripts':
            'kaidoku = kaidoku.main:main'
    },
)
